package broker

import (
	"github.com/pivotal-cf/brokerapi"
	"github.com/pivotal-cf/on-demand-services-sdk/serviceadapter"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
)

func (b *Broker) Bind(instanceID, bindingID string, details brokerapi.BindDetails) (brokerapi.Binding, error) {
	logger := b.loggerFactory.NewLogger()

	errs := func(context string, err error) (brokerapi.Binding, error) {
		logger.Printf("error %s: %s", context, err)
		return brokerapi.Binding{}, convertToDisplayableError(err, logger)
	}

	vms, manifest, err := b.getDeploymentInfo(instanceID, logger)
	switch err.(type) {
	case boshclient.DeploymentNotFoundError:
		return errs("error binding: instance "+instanceID+", not found", DisplayableError{brokerapi.ErrInstanceDoesNotExist})
	case error:
		return errs("gathering binding info", err)
	}

	logger.Printf("service adapter will create binding with ID %s for instance %s\n", bindingID, instanceID)
	mappedParams, err := convertToMap(details)
	if err != nil {
		return errs("converting to map", err)
	}
	adapterBinding, err := b.serviceAdapter.CreateBinding(bindingID, vms, manifest, mappedParams, logger)
	switch err := err.(type) {
	case serviceadapter.BindingAlreadyExistsError:
		logger.Printf("error: %s\n", err)
		return errs("creating binding", DisplayableError{brokerapi.ErrBindingAlreadyExists})
	case serviceadapter.AppGuidNotProvidedError:
		logger.Printf("error: %s\n", err)
		return errs("creating binding", DisplayableError{brokerapi.ErrAppGuidNotProvided})
	case error:
		return errs("creating binding", err)
	}

	return brokerapi.Binding{
		Credentials:     adapterBinding.Credentials,
		SyslogDrainURL:  adapterBinding.SyslogDrainURL,
		RouteServiceURL: adapterBinding.RouteServiceURL,
	}, nil
}
